(ns wanijo.framework.auth
  (:require [compojure.core :refer [POST GET defroutes]]
            [ring.util.response :refer [redirect]]
            [ring.util.anti-forgery :refer [anti-forgery-field]]
            [buddy.hashers :as hashers]
            [hiccup.form :as hform]
            [wanijo.framework.view :as view]
            [wanijo.user.domain :as user-domain]))

(defn- login-check! [req]
  (let [{{:keys [uname pw]} :params} req
        unode (user-domain/find! uname)
        pwmatch (when-let [hash (:pw unode)]
                  (hashers/check pw hash))]
    (if pwmatch
      (-> (redirect "/")
          (assoc-in [:session :ident] uname)
          (assoc-in [:session :uuid] (:uuid unode)))
      (-> (redirect "/login")
          (assoc :flash :invalid-credentials)))))

(defn login! [req]
  (view/layout!
   :content
   [[:h1 "Kama ken"]
    (hform/form-to
     [:post "/login-check"]
     (when (:flash req) [:section.flash (:flash req)])
     ;;
     (hform/label "uname" "Nimi")
     (hform/text-field {:required "required"} "uname")
     ;;
     (hform/label "pw" "Toki Pimeja")
     (hform/password-field {:required "required"} "pw")
     ;;
     (hform/submit-button "Kama")
     (anti-forgery-field))]))

(defroutes routes
  (GET "/login" [] login!)
  (POST "/login-check" [] login-check!)
  (GET "/logout" [] (-> (redirect "/login")
                        (assoc :session nil))))
