(ns wanijo.schema.routes
  (:require [compojure.core :refer [defroutes GET POST DELETE]]
            [ring.util.response :as resp]
            [wanijo.framework.view :as view]
            [wanijo.framework.form :as form]
            [wanijo.schema.domain :as domain]
            [wanijo.schema.view :as view-schema]
            [wanijo.attribute.domain :as attr-domain]))

(defn new! [req]
  (if (form/valid? view-schema/new-form req)
    (do
      (domain/create-new!
       (get-in req [:params :schema-name])
       (get-in req [:session :uuid]))
      (resp/redirect "/schema"))
    (view-schema/overview! req)))

(defn show-schema! [uuid session]
  (view-schema/show-schema!
   (domain/find-by-uuid! uuid)
   (attr-domain/find-by-schema! uuid)
   session))

(defn delete-schema! [uuid session]
  (if (domain/can-user-modify? uuid (:uuid session))
    (do
      (domain/delete! uuid)
      (resp/redirect "/schema"))
    {:status 403}))

(defroutes routes
  (GET "/schema" [] view-schema/overview!)
  (GET "/schema/:uuid" [uuid :as req] (show-schema! uuid (:session req)))
  (POST "/schema/new" [] new!)
  (DELETE "/schema/:uuid/delete" [uuid :as req] (delete-schema! uuid (:session req))))
